Attribute VB_Name = "WitnessControl"
Option Explicit
Dim gobjWitness As Object   'Witness object
'Function to open witness and run the model for the specified length of time
Public Sub RunWitnessModel(nRunLength As Integer)
    On Error GoTo RunWitnessModel_Err
    
    Dim xlApp As Excel.Application
    
    'set the cursor to busy
    frmMain.MousePointer = vbHourglass
    
    'open the Excel spreadsheet to prevent any macro
    'messages and to enable us to save it
    Set xlApp = OpenSpreadsheet(False)
    'check that we have been able to open the spreadsheet
    If Not xlApp Is Nothing Then
        'open witness
        If OpenWitness() Then
            'load the witness model
            If LoadWitnessModel() Then
                'run the model for the specified length of time
                RunModel CStr(nRunLength)
                'save and close the spreadsheet
                If Not xlApp Is Nothing Then
                    If xlApp.Workbooks.Count > 0 Then
                        xlApp.ActiveWorkbook.Save
                    End If
                    xlApp.Quit
                    Set xlApp = Nothing
                End If
                'reset witness back to time 0
                ResetWitness
                'close witness
                CloseWitness False
                frmMain.Visible = True
            End If
        End If
    End If
    'reset the cursor back to the system default
    frmMain.MousePointer = vbDefault

RunWitnessModel_Exit:
    Exit Sub

RunWitnessModel_Err:
    MsgBox Error$
    Resume Next
End Sub
Private Function OpenWitness() As Boolean
    On Error GoTo OpenWitness_ValidToContinue
    
    OpenWitness = True 'assume it will succeed
    
    ' Detect whether Witness is currently open
    gobjWitness = GetObject(, "Witness.WCL")
        
    'successful connection - Witness is already open.  Exit now
    OpenWitness = True
    
    Exit Function
    
OpenWitness_ValidToContinue:
    'clear the active error
    Resume OpenWitness_Continue

OpenWitness_Continue:
    On Error GoTo OpenWitness_WaitingToLoad
    
    ' Witness is not active - it is valid to continue
    
    ' Kick off WITNESS
    If ViewWitness() Then
        Shell App.Path & "\..\witness.exe", vbMaximizedFocus
    Else
        Shell App.Path & "\..\witness.exe", vbMinimizedNoFocus
    End If
    
    ' Give WITNESS time to load
OpenWitness_WaitingToLoadResume:
    DoEvents
    ' Connect to WITNESS object
    Set gobjWitness = GetObject(, "Witness.WCL")
    
    On Error GoTo 0
    ' Wait for open sequence to complete
    Do
    Loop While gobjWitness.ModelStatus() = 3
    
    OpenWitness = True
    Exit Function
    
OpenWitness_WaitingToLoad:
    Resume OpenWitness_WaitingToLoadResume
End Function

Private Sub RunModel(strRunLength As String)
    On Error GoTo RunModel_Err
    
    gobjWitness.Run (strRunLength)
    
    If gobjWitness.ModelStatus = 4 Then 'there is an error
        HandleWitnessError 'get error info and close witness
    End If
    
    Do While gobjWitness.ModelStatus = 2 And gobjWitness.variable("TIME") = 0
        If gobjWitness.ModelStatus = 4 Then 'there is an error
            HandleWitnessError 'get error info and close witness
        End If
        DoEvents
    Loop
    
    If gobjWitness.ModelStatus = 4 Then 'there is an error
        HandleWitnessError 'get error info and close witness
    End If
    
    Do While gobjWitness.ModelStatus <> 2
        If gobjWitness.ModelStatus = 4 Then 'there is an error
            HandleWitnessError 'get error info and close witness
        End If
        DoEvents
    Loop
    
    If gobjWitness.ModelStatus = 4 Then 'there is an error
        HandleWitnessError 'get error info and close witness
    End If
    
    Exit Sub

RunModel_Exit:
    Exit Sub
    
RunModel_Err:
    MsgBox ("Error running Witness to " & strRunLength)
    Resume RunModel_Exit
End Sub

'function to load the witness model
Private Function LoadWitnessModel() As Boolean
On Error GoTo NoObject
    gobjWitness.BeginOLE

On Error GoTo LoadWitness_Error
    'load the model
    gobjWitness.wcl ("LOAD ContactCenter")
    'wait for the load process to complete
    Do
    Loop While gobjWitness.ModelStatus = 3
    gobjWitness.EndOLE
    LoadWitnessModel = True
    
    Exit Function

LoadWitness_Error:
    gobjWitness.EndOLE

NoObject:
    MsgBox ("Error loading model")
    LoadWitnessModel = False
End Function

'function to reset the witness model to time 0
Private Sub ResetWitness()
    On Error GoTo ResetWitness_Err

    If gobjWitness.ModelStatus = 2 Then
        gobjWitness.Begin
    Else
        MsgBox "Unable to reset Witness.", vbOKOnly
    End If
    
    Exit Sub
    
ResetWitness_Exit:
    Exit Sub
    
ResetWitness_Err:
    MsgBox ("Error resetting Witness.")
End Sub

Private Sub CloseWitness(bSavePrompt As Boolean, Optional bSave As Boolean)
    On Error GoTo CloseWitness_Err
    
    ' Activate Witness application
    AppActivate ("WITNESS")
    ' send Alt - F4
    SendKeys "%{F4}"
    ' Handle save dialog if necessary
    If Not IsMissing(bSave) Then
        If bSave Then
            SendKeys "Y", True
        Else
            SendKeys "N", True
        End If
    End If
    Set gobjWitness = Nothing

CloseWitness_Exit:
    Exit Sub

CloseWitness_Err:
    If Err.Number > 0 Then
        MsgBox Error$
    End If
    Resume Next
End Sub

Private Sub HandleWitnessError()
    On Error GoTo HandleWitnessError_Err
    
    'report that there was an error running Witness
    Dim Ret As Integer
    Dim ErrorText As String
    
    Ret = MsgBox("An error occurred whilst running the simulation." & Chr(13) & Chr(10) & "Do you wish to view details?", vbYesNo)
    
    If Ret = 6 Then 'user has chosen "yes"
        ErrorText = gobjWitness.LastError 'Get the error report
        Ret = MsgBox(ErrorText, vbOKOnly) 'display error
    End If
    CloseWitness False, False
    End

HandleWitnessError_Exit:
    Exit Sub

HandleWitnessError_Err:
    MsgBox Error$
    Resume Next
End Sub

Public Function ViewWitness() As Boolean
    On Error GoTo ViewWitness_Err

    ViewWitness = frmMain.chkWitnessVisible

ViewWitness_Exit:
    Exit Function

ViewWitness_Err:
    MsgBox Error$
    Resume Next
End Function

'function to open the Excel spreadsheet
Public Function OpenSpreadsheet(bMaximised As Boolean) As Excel.Application
    On Error GoTo OpenSpreadsheet_Err

    Dim xlApp As Excel.Application
    
    'create a new instance of Excel
    Set xlApp = New Excel.Application
    'check that an instance of Excel has successfully been created
    If Not xlApp Is Nothing Then
        'open the spreadsheet
        xlApp.Workbooks.Open App.Path & "\ContactCenter.xls"
        'select the viewing method
        If bMaximised Then
            xlApp.WindowState = xlMaximized
        Else
            xlApp.WindowState = xlMinimized
        End If
        xlApp.Visible = True
    End If

    'return the handle to the Excel instance
    Set OpenSpreadsheet = xlApp
    
OpenSpreadsheet_Exit:
    Exit Function

OpenSpreadsheet_Err:
    If Not xlApp Is Nothing Then
        xlApp.Quit
    End If
    MsgBox Error$
    Resume Next
End Function
